#ifndef WELDINGMACHINE_HPP
#define WELDINGMACHINE_HPP

#include <memory>

// 焊机类型
enum WeldingMachineType {
    WeldingMachineType_AoTai,

    WeldingMachineType_Unknown,
};

// 工作模式
enum WorkingMode {
    WorkingMode_Standard = 0,   // 平特性
    WorkingMode_PulsedArc,      // 脉冲程序
    WorkingMode_JobMode,        // 调用状态
    WorkingMode_Separation,     // 分别模式
};

// 错误码
enum ErrorCode {
    ErrorCode_OK = 0,
    ErrorCode_InvalidDeviceType = -1,   // 无效的设备类型
    ErrorCode_NoObjectCreated = -2,     // 没有创建对象
    ErrorCode_ConnectionFailure = -3,   // 连接失败
    ErrorCode_Unconnected = -4,         // 未连接
    ErrorCode_ModbusSendError = -5,     // modbus发送失败
    ErrorCode_UnknownMode = -6,         // 对当前品牌来说，未知的工作模式
    ErrorCode_WrongValue = -7,          // 设置的值有问题
};

/**
 * @brief      日志输出对应的回调函数类型
 * @param logLevel  日志级别 0~5分别是INFO, DEBUG, WARN, ERROR, FATAL
 * @param str       日志信息;
 */
typedef void (*RobotLogPrintCallback)(int logLevel, const char *str);

class WeldingMachine
{
public:
    WeldingMachine();

    /**
     * @brief 注册日志回调函数
     * @param callback 回调函数
     * @note 多个实例时可以只调用一次
     */
    static void registerLogPrintCallback(RobotLogPrintCallback callback);

    /**
     * @brief 创建焊机对象
     * @param type 焊机类型
     * @return 成功返回0，失败返回错误码
     */
    int createWeldingMachine(const WeldingMachineType type);

    /**
     * @brief rtu连接
     * @param device 串口名
     * @param baud  波特率
     * @param parity 校验位
     * @param data_bit 数据位
     * @param stop_bit 停止位
     * @return 连接成功返回0，失败返回错误码
     */
    int connect(const std::string &device, const int &baud, const std::string &parity,
                const int &data_bit, const int &stop_bit, const int &id);

    /**
     * @brief 以太网连接
     * @param ip
     * @param port
     * @return 连接成功返回0，失败返回错误码
     */
    int connect(const std::string &ip, const int &port, const int &id);

    /**
     * @brief 断开连接
     * @return
     */
    int disconnect();

    /**
     * @brief 获取焊机类型
     * @return 返回焊机类型
     */
    WeldingMachineType type();

    /**
     * @brief 设置工作模式
     * @param working_mode 工作模式
     * @return 接口调用成功与否，成功返回0，失败返回错误码
     */
    int setWorkingMode(const WorkingMode &working_mode);

    /**
     * @brief 获取工作模式
     * @param working_mode 工作模式
     * @return 接口调用成功与否，成功返回0，失败返回错误码
     */
    int getWorkingMode(WorkingMode &working_mode);

    /**
     * @brief 通知焊机准备好
     * @return 接口调用成功与否，成功返回0，失败返回错误码
     */
    int robotReady();

    /**
     * @brief 机器人是否准备就绪
     * @param is_ready 就绪状态
     * @return 接口调用成功与否，成功返回0，失败返回错误码
     */
    int robotIsReady(bool &is_ready);

    /**
     * @brief 开始焊接/起弧
     * @return 接口调用成功与否，成功返回0，失败返回错误码
     */
    int startWelding();

    /**
     * @brief 停止焊接/熄弧
     * @return 接口调用成功与否，成功返回0，失败返回错误码
     */
    int stopWelding();

    /**
     * @brief 起弧成功
     * @param is_stable 起弧状态
     * @return 接口调用成功与否，成功返回0，失败返回错误码
     */
    int arcStable(bool &is_stable);

    /**
     * @brief 开始送气
     * @return 接口调用成功与否，成功返回0，失败返回错误码
     */
    int startGas();

    /**
     * @brief 停止送气
     * @return 接口调用成功与否，成功返回0，失败返回错误码
     */
    int stopGas();

    /**
     * @brief 气体检测
     * @param status 气体检测结果
     * @return 接口调用成功与否，成功返回0，失败返回错误码
     */
    int gasDetection(bool &status);

    /**
     * @brief 开始送丝
     * @return 接口调用成功与否，成功返回0，失败返回错误码
     */
    int startWireFeed();

    /**
     * @brief 结束送丝
     * @return 接口调用成功与否，成功返回0，失败返回错误码
     */
    int stopWireFeed();

    /**
     * @brief 送丝检测
     * @param status 送丝检测结果
     * @return 接口调用成功与否，成功返回0，失败返回错误码
     */
    int wireFeedDetection(bool &status);

    /**
     * @brief 开始回抽丝
     * @return 接口调用成功与否，成功返回0，失败返回错误码
     */
    int startWireFeedBack();

    /**
     * @brief 结束回抽丝
     * @return 接口调用成功与否，成功返回0，失败返回错误码
     */
    int stopWireFeedBack();

    /**
     * @brief 回抽丝检测
     * @param status 回抽丝检测结果
     * @return 接口调用成功与否，成功返回0，失败返回错误码
     */
    int wireFeedBackDetection(bool &status);

    /**
     * @brief 给定电流
     * @param current 电流，单位A
     * @return 接口调用成功与否，成功返回0，失败返回错误码
     */
    int givenCurrent(const double &current);

    /**
     * @brief 给定电压
     * @param voltages 电压，单位V
     * @return 接口调用成功与否，成功返回0，失败返回错误码
     */
    int givenVoltage(const double &voltages);

    /**
     * @brief 实时电流
     * @param current 电流，单位A
     * @return 接口调用成功与否，成功返回0，失败返回错误码
     */
    int weldingCurrent(double &current);

    /**
     * @brief 实时电压
     * @param voltages 电压，单位V
     * @return 接口调用成功与否，成功返回0，失败返回错误码
     */
    int weldingVoltage(double &voltages);

    /**
     * @brief 焊机故障恢复
     * @return 接口调用成功与否，成功返回0，失败返回错误码
     */
    int sourceErrorReset();

protected:
    void *d_;
};

using WeldingMachinePtr = std::shared_ptr<WeldingMachine>;

#endif // WELDINGMACHINE_HPP
